/** @file
 *  @brief		SmartCollision Hello application.
 *
 *	@author		3D Incorporated
 *	@date		2005/11/11
 *
 */
#include <tchar.h>
#include <stdio.h>
#include "sc.h"

double vertices[12]={0,0,0,
		     1,0,0,
		     0,1,0,
                     0,0,1};
int triangles[12]={0,2,1,
                   1,3,0,
		   0,3,2,
		   1,2,3};

bool informErr(int result)
{
	if(result!=0)
	{
		if(result==300)
			printf("Error: License Failure. Please check your license.\n",result);
		else
			printf("Error:AddTriangles failed(%d)\n",result);

		getchar();
		return false;
	}
	else
		return true;
}


int _tmain(int argc, _TCHAR* argv[])
{
    SCSceneManager manager(SC_SCENE_MANAGER_CLOSED_POLYHEDRA);
    SCObject object1,object2;
    double position1[3]={1.0,1.0,1};
    double position2[3]={0.0,0.0,0.5};
    double distance;
	if(object1.AddTriangles(SC_OBJECT_TYPE_CLOSED_POLYHEDRON,vertices,4,triangles,4)
		==SC_ERROR_INVALID_LICENSE){
			printf("Error: Invalid lincense.\n");
			return -1;
	}
    object2.AddTriangles(SC_OBJECT_TYPE_CLOSED_POLYHEDRON,vertices,4,triangles,4);
    manager.SetAttributeDouble(SC_SCENE_MANAGER_MAX_DISTANCE,10.0);
    manager.SetAttributeDouble(SC_SCENE_MANAGER_TOLERANCE,0.1);
    manager.SetAttributeInteger(SC_SCENE_MANAGER_MAX_ITERATION,100);
    int iRes =manager.AddObject(1000,&object1);
	if(!informErr(iRes))
		return -1;

    manager.AddObject(2000,&object2);
    manager.AddControlledObject(2000);
    printf("Hellow SmartCollision!\n");
    manager.SetTransformation(SC_POSITION_ORIGIN,position1);
    manager.UpdateStatus();

    manager.GetStatus(SC_DISTANCE,&distance);
    printf("In position=%lg,%lg,%lg\n",position1[0],position1[1],position1[2]);
    printf("  Distance=%lg(=sqrt(2/3))\n",distance);
    manager.SetTransformation(SC_POSITION_ORIGIN,position2);
    manager.UpdateStatus();
    manager.GetStatus(SC_DISTANCE,&distance);
    printf("In position=%lg,%lg,%lg\n",position2[0],position2[1],position2[2]);
    printf("  Pentration depth=%lg(=sqrt(1/12))\n",-distance);

	getchar();
    return 0;
}


