#ifndef BASIC_TYPES_H
#define BASIC_TYPES_H

#include <math.h>

#ifndef M_PI
#define	M_PI		3.14159265358979323846
#endif

typedef int BEnum;
typedef float BReal;

template<class T>
class BTVector3{
public:
    T x,y,z;

    BTVector3(void){}
    BTVector3(T x0,T y0,T z0){	x=x0;y=y0;z=z0;}
	BTVector3(const T*p){		x=p[0];y=p[1];z=p[2];}

    void Set(T x0,T y0,T z0){	x=x0;y=y0;z=z0;}
    void GetValue(float*t){		t[0]=x;t[1]=y;t[2]=z;}
    void GetValue(double*t){	t[0]=x;t[1]=y;t[2]=z;}

    T Norm(void)const{			return (T)sqrt(x*x+y*y+z*z);}

	T operator[](int i)const{				return (&x)[i];}
	void operator+=(const BTVector3&a){		x+=a.x;y+=a.y;z+=a.z;}
	void operator-=(const BTVector3&a){		x-=a.x;y-=a.y;z-=a.z;}
	bool operator==(const BTVector3&a){		return x==a.x&&y==a.y&&z==a.z;}
	bool operator!=(const BTVector3&a){		return x!=a.x||y!=a.y||z!=a.z;}

	BTVector3 operator+(const BTVector3&a) const{	return BTVector3(x+a.x,y+a.y,z+a.z);}
	BTVector3 operator-(const BTVector3&a) const{	return BTVector3(x-a.x,y-a.y,z-a.z);}
	BTVector3&operator=(const BTVector3&a){	x=a.x;y=a.y;z=a.z;return*this;}
	BTVector3&operator=(int a){				x=y=z=(T)a;return*this;}
	BTVector3&operator=(float a){			x=y=z=(T)a;return*this;}
	BTVector3&operator=(double a){			x=y=z=(T)a;return*this;}
	BTVector3&operator=(const float a[3]){	x=a[0];y=a[1];z=a[2];return*this;}
	BTVector3&operator=(const double a[3]){	x=static_cast<BReal>(a[0]);y=static_cast<BReal>(a[1]);z=static_cast<BReal>(a[2]);return*this;}

	void operator/=(T a){					a=((T)1.0)/a;x*=a;y*=a;z*=a;}
	void operator*=(T a){					x*=a;y*=a;z*=a;}
	operator T*(void)const{return(T*)&x;}

};
typedef BTVector3<BReal> BVector3;

inline BVector3 operator*(const BVector3&t,BReal a)
{
    return BVector3(a*t.x,a*t.y,a*t.z);
}

class BQuaternion{
private:
	BReal s;
	BReal vx,vy,vz;

	BReal Norm(void)const{return (BReal)sqrt(s*s+vx*vx+vy*vy+vz*vz);}

public:
	BQuaternion(void){;}

	void Identity(void){s=1;vx=vy=vz=0;}
    void Normalize(void){BReal t;t=(BReal)1.0/Norm();s*=t;vx*=t;vy*=t;vz*=t;}
	void imul(const BQuaternion&a)
	{
		BQuaternion b;
		b=*this;
		s=a.s*b.s-(a.vx*b.vx+a.vy*b.vy+a.vz*b.vz);
		vx=a.s*b.vx+b.s*a.vx+(a.vy*b.vz-a.vz*b.vy);
		vy=a.s*b.vy+b.s*a.vy+(a.vz*b.vx-a.vx*b.vz);
		vz=a.s*b.vz+b.s*a.vz+(a.vx*b.vy-a.vy*b.vx);
    }

	void Set(BReal s0,BReal vx0,BReal vy0,BReal vz0){s=s0;vx=vx0;vy=vy0;vz=vz0;}
	void SetRotation(BReal wx,BReal wy,BReal wz)
	{
		BReal c,s,w;
		w=(BReal)sqrt(wx*wx+wy*wy+wz*wz);
		c=(BReal)cos(w*0.5);
		s=(BReal)sin(w*0.5);
		if(w>0)
			s/=w;
		Set(c,s*wx,s*wy,s*wz);
	}
    void SetXYZAngle(BReal ax,BReal ay,BReal az)
	{
		BQuaternion dq;
		SetRotation(ax,0,0);
		dq.SetRotation(0,ay,0);(*this)*=dq;
		dq.SetRotation(0,0,az);(*this)*=dq;
    }

    void SetYXZAngle(BReal ay,BReal ax,BReal az)
	{
		BQuaternion dq;
		SetRotation(0,ay,0);
		dq.SetRotation(ax,0,0);(*this)*=dq;
		dq.SetRotation(0,0,az);(*this)*=dq;
    }
    void SetMatrix0(BReal m[9])
	{
		BReal f00,f01,f02;
		BReal f12,f22;
		BReal f10,f11;
		f00=m[0];f01=m[1];f02=m[2];
		f12=m[5];
		f22=m[8];
		f10=m[3];f11=m[4];
		BReal a[3];
		if(f02<-1.0)
			f02=(BReal)-1.0;
		else if(f02>1.0)
			f02=(BReal)1.0;
		if(f02==1)
		{
			a[1]=(BReal)(M_PI*0.5);
			a[0]=(BReal)atan2(f10,f11);
			a[2]=0;
		}
		else if(f02==-1)
		{
			a[1]=-(BReal)(M_PI*0.5);
			a[0]=(BReal)atan2(-f10,f11);
			a[2]= 0;	    
		}
		else
		{
			a[0]=(BReal)atan2(-f12,f22);//ax
			//a[1]=asin(f02);      //ay
			a[2]=(BReal)atan2(-f01,f00);//az
			BReal t=(BReal)cos(a[2]);
			if(t>0)a[1]=(BReal)(M_PI*0.5)-(BReal)atan2(f00,f02*t);  //ay
			else   a[1]=(BReal)(M_PI*0.5)-(BReal)atan2(-f00,-f02*t);//ay
		}
		SetXYZAngle(a[0],a[1],a[2]);
    }

	void GetValue(float*t){t[0]=s;t[1]=vx;t[2]=vy;t[3]=vz;}
	void GetValue(double*t){t[0]=s;t[1]=vx;t[2]=vy;t[3]=vz;}
    void GetYXZAngle(BReal a[3])const
	{
		BReal f10,f11,f12;
		BReal f02,f22;

		f10=(vy*vx+s*vz)*2;//a[3]
		f11=(BReal)1.0-(vz*vz+vx*vx)*2;//a[4]
		f12=(vy*vz-s*vx)*2;//a[5]
		//a[2],a[5],a[8]
		f02=(vx*vz+s*vy)*2;//a[2]
		f22=(BReal)1.0-(vx*vx+vy*vy)*2;//a[8]
		{
			a[0]=(BReal)atan2(f02,f22);//ax,cos(a2)>0
			a[2]=(BReal)atan2(f10,f11);//az,cos(a2)>0
			a[1]=(BReal)asin(-f12);
		}
    }
    void GetMatrix(double a[9])const
	{
		a[0]=1.0-2.0*(vy*vy+vz*vz);
		a[1]=2.0*(vx*vy-s*vz);
		a[2]=2.0*(vx*vz+s*vy);
		a[3]=2.0*(vy*vx+s*vz);
		a[4]=1.0-2.0*(vz*vz+vx*vx);
		a[5]=2.0*(vy*vz-s*vx);
		a[6]=2.0*(vz*vx-s*vy);
		a[7]=2.0*(vz*vy+s*vx);
		a[8]=1.0-2.0*(vx*vx+vy*vy);
    }
    void GetMatrix(float a[9])const
	{
		a[0]=1.0f-2.0f*(vy*vy+vz*vz);
		a[1]=2.0f*(vx*vy-s*vz);
		a[2]=2.0f*(vx*vz+s*vy);
		a[3]=2.0f*(vy*vx+s*vz);
		a[4]=1.0f-2.0f*(vz*vz+vx*vx);
		a[5]=2.0f*(vy*vz-s*vx);
		a[6]=2.0f*(vz*vx-s*vy);
		a[7]=2.0f*(vz*vy+s*vx);
		a[8]=1.0f-2.0f*(vx*vx+vy*vy);
    }

	void operator*=(const BQuaternion&b)
	{
		BQuaternion a;
		a=*this;
		s=a.s*b.s-(a.vx*b.vx+a.vy*b.vy+a.vz*b.vz);
		vx=a.s*b.vx+b.s*a.vx+(a.vy*b.vz-a.vz*b.vy);
		vy=a.s*b.vy+b.s*a.vy+(a.vz*b.vx-a.vx*b.vz);
		vz=a.s*b.vz+b.s*a.vz+(a.vx*b.vy-a.vy*b.vx);
	}
	BQuaternion&operator=(const float a[4])
	{	
		s =(BReal)a[0];	
		vx=(BReal)a[1];	
		vy=(BReal)a[2];	
		vz=(BReal)a[3];	
		return*this;
	}
	BQuaternion&operator=(const double a[4])
	{
		s =(BReal)a[0];
		vx=(BReal)a[1];
		vy=(BReal)a[2];
		vz=(BReal)a[3];
		return*this;
	}
};

class CoordinateMatrix
{
private:
    BReal mxx,mxy,mxz;
    BReal myx,myy,myz;
    BReal mzx,mzy,mzz;
    BReal tx,ty,tz;

public:
	void InverseRotate(BVector3&d,const BVector3&s)const
	{
		d.x=mxx*s.x+myx*s.y+mzx*s.z;
		d.y=mxy*s.x+myy*s.y+mzy*s.z;
		d.z=mxz*s.x+myz*s.y+mzz*s.z;
	};

	void Convert(BVector3&a,const BVector3&b)const
	{
		a.x=mxx*b.x+mxy*b.y+mxz*b.z+tx;
		a.y=myx*b.x+myy*b.y+myz*b.z+ty;
		a.z=mzx*b.x+mzy*b.y+mzz*b.z+tz;
	};

	void Set(const BQuaternion&q)
	{
		q.GetMatrix(&mxx);
		tx=0;ty=0;tz=0;
	}
	void Set(const BVector3&t,const BQuaternion&q)
	{
		BReal rot[3*3];
		q.GetMatrix(rot);
		mxx=rot[0];mxy=rot[1];mxz=rot[2];
		myx=rot[3];myy=rot[4];myz=rot[5];
		mzx=rot[6];mzy=rot[7];mzz=rot[8];
		tx=t.x;ty=t.y;tz=t.z;
	}

    void GetGLMatrix(double*m)const
	{
		m[0]=mxx;m[4]=mxy;m[ 8]=mxz;m[12]=tx;
		m[1]=myx;m[5]=myy;m[ 9]=myz;m[13]=ty;
		m[2]=mzx;m[6]=mzy;m[10]=mzz;m[14]=tz;
		m[3]=0.0;m[7]=0.0;m[11]=0.0;m[15]=1.0;
    }
    void GetGLMatrix(float*m)const
	{
		m[0]=mxx;m[4]=mxy;m[ 8]=mxz;m[12]=tx;
		m[1]=myx;m[5]=myy;m[ 9]=myz;m[13]=ty;
		m[2]=mzx;m[6]=mzy;m[10]=mzz;m[14]=tz;
		m[3]=0.0f;m[7]=0.0f;m[11]=0.0f;m[15]=1.0f;
    }

};

#endif
