#ifndef _EPHANTOM_H_
#define _EPHANTOM_H_

/** @file
 *  @brief This file is Definition of Class EPhantom.
 */
#include "EasyDeviceImpl.h"

class EPhantom : public EasyDevice::Impl
{
private:
	BVector3 center_;
	BVector3 origin_;
	BVector3 scale_;

	BQuaternion orientation_;
	BQuaternion orientation0_;
	BQuaternion centerOrientation_;
	BQuaternion centerOrientation0_; 

	BVector3 position_;
	BVector3 position0_;

	int last_state_;

	void Convert(BVector3&t)
	{
	t.x=origin_.x+t.x*scale_.x;
	t.y=origin_.y+t.y*scale_.y;
	t.z=origin_.z+t.z*scale_.z;
	}
	void _GetOrientation(BQuaternion&q);
	void _GetOrientation(BQuaternion&q, BQuaternion&c0);

public:
	EPhantom();
	~EPhantom();

	void SetForce(BVector3&t);
	void SetTorque(BVector3&t);
	int  GetButton(int n);
	void WaitForUserAdmit();

	void CallBackHpFunction(void);
	void CallBackStFunction(void);
	void CallBackKbFunction(void);

	void SetScale(BReal s);
	void SetOrientation(const BQuaternion&orientation);

	void GetPosition(BVector3&position);
	void GetOrientation(BQuaternion&orientation);
	void GetLastPosition(BVector3&p)const{p=position_;}
	void GetLastOrientation(BQuaternion&q)const{q=orientation_;}

	bool Initialize(void);
	bool Stop(void);

	void SetCallBackHpFunction(void(*hp)(void));
	void SetCallBackStFunction(void(*st)(void));
	void SetCallBackKbFunction(void(*kb)(unsigned char), unsigned char key);
};

#endif