/** @file
 *  @brief This file is implimentaion of class EMouse.
 */
#include "EMouse.h"
#include <windows.h>

#define HDM_LEFT_BUTTON   1
#define HDM_RIGHT_BUTTON  2
#define HDM_MIDDLE_BUTTON 3
#define HDM_DOWN 11
#define HDM_UP   12

static const BReal frequency=1000;
static const BReal defaultscale=1.0;

static MMRESULT				TimerId;
static CRITICAL_SECTION		GlobalCriticalSection;

void CALLBACK mm_timer_func(UINT uiID, UINT uiNo, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	EMouse* me = reinterpret_cast<EMouse*>(dwUser);
	if(me)
		me->CallBackHpFunction();

	return;
}

EMouse::EMouse()
{
	center_=0;
	origin_=0;
	scale_.x=defaultscale;
	scale_.y=defaultscale;
	scale_.z=defaultscale;

	orientation_.Identity();
	orientation0_.Identity();
	centerOrientation_.Identity();
	centerOrientation0_.Identity();

	position_=0;
	position0_=0;

	initFlag_=0;

	objectManipulator.Initialize();
	objectManipulator.SetOrientation(centerOrientation_);

	InitializeCriticalSection(&GlobalCriticalSection);
}

EMouse::~EMouse()
{
}

void 
EMouse::_GetOrientation(BQuaternion&q)
{
	objectManipulator.GetOrientation(q);
}

void 
EMouse::_GetOrientation(BQuaternion&q,
			   BQuaternion&c0)
{
	_GetOrientation(q);
	BReal a[3],b[3];
	q.GetYXZAngle(a);
	c0.GetYXZAngle(b);
	q.SetYXZAngle(a[0]-b[0],
			a[1]-b[1],
			a[2]-b[2]);
}

void 
EMouse::SetScale(BReal s)
{
	scale_.x=s;
	scale_.y=s;
	scale_.z=s;

	return;
}

void
EMouse::SetOrientation(const BQuaternion& q)
{
	BQuaternion t;
	orientation0_=q;
	_GetOrientation(t,centerOrientation_);
	centerOrientation0_=t;
	centerOrientation0_.Normalize();

	return;
}

void 
EMouse::GetPosition(BVector3&position)
{
	BVector3 p0;
	objectManipulator.GetPosition(p0);

	position.x=p0.x-center_.x;
	position.y=p0.y-center_.y;
	position.z=p0.z-center_.z;

	Convert(position);

	position0_=position_;
	position_=position;

	return;
}

void 
EMouse::GetOrientation(BQuaternion& q)
{
	BQuaternion t;
	q.Identity();
	_GetOrientation(t,centerOrientation_);
	q=t;
	{
		CoordinateMatrix t;
		BVector3 ex,ey,ez;
		ex.Set(1,0,0);
		ey.Set(0,1,0);
		ez.Set(0,0,1);
		BVector3 fx,fy,fz;
		t.Set(orientation0_);
		t.Convert(fx,ex);
		t.Convert(fy,ey);
		t.Convert(fz,ez);
		t.Set(centerOrientation0_);
		t.InverseRotate(ex,fx);
		t.InverseRotate(ey,fy);
		t.InverseRotate(ez,fz);
		t.Set(q);
		t.Convert(fx,ex);
		t.Convert(fy,ey);
		t.Convert(fz,ez);

		ex=fx;ey=fy;ez=fz;
		BReal m[9];
		m[0]=ex.x;m[3]=ex.y;m[6]=ex.z;
		m[1]=ey.x;m[4]=ey.y;m[7]=ey.z;
		m[2]=ez.x;m[5]=ez.y;m[8]=ez.z;
		q.SetMatrix0(m);
	}
	q.Normalize();
	orientation_=q;

	return;
}

bool 
EMouse::Initialize(void)
{
	initFlag_=1;
	BReal t=1000.0f/frequency;
	if(t<1)
		t=1;
	TimerId=timeSetEvent(static_cast<UINT>(t),
				1,mm_timer_func,
				(DWORD)this,
				TIME_PERIODIC|TIME_CALLBACK_FUNCTION);

	return true;
}

bool 
EMouse::Stop(void)
{
	initFlag_=0;
	timeKillEvent(TimerId);
	Sleep(10);
	DeleteCriticalSection(&GlobalCriticalSection);

	return true;
}

void
EMouse::CallBackHpFunction(void)
{
	if(!initFlag_)
		return;

	Enter();
	if(callBackHpFunction)
		callBackHpFunction();
	Leave();

	return;
}

void 
EMouse::SetCallBackHpFunction(void(*hp)(void))
{
	callBackHpFunction=hp;
	return;
}

void 
EMouse::SetCallBackStFunction(void(*st)(void))
{
	Enter();
	st();
	Leave();

	return;
}

void 
EMouse::SetCallBackKbFunction(void(*kb)(unsigned char), unsigned char key)
{
	Enter();
	kb(key);
	Leave();

	return;
}

void 
EMouse::MouseFunction(int button,int state, int x, int y)
{
	ObjectManipulator* o = &objectManipulator;

	if(o==0)
		return;
	if(button==HDM_LEFT_BUTTON)
	{
		if(state==HDM_DOWN)
			o->OnLeftButtonDown(x,y);
		else if(state==HDM_UP)
			o->OnLeftButtonUp(x,y);
	}
	else if(button==HDM_RIGHT_BUTTON)
	{
		if(state==HDM_DOWN)
			o->OnRightButtonDown(x,y);
		else if(state==HDM_UP)
			o->OnRightButtonUp(x,y);
	}
	else if(button==HDM_MIDDLE_BUTTON)
	{
		if(state==HDM_DOWN)
			o->OnMiddleButtonDown(x,y);
		else if(state==HDM_UP)
			o->OnMiddleButtonUp(x,y);
	}

	return;
}

void 
EMouse::MotionFunction(int x, int y)
{
	ObjectManipulator* o = &objectManipulator;

	if(o==0)
		return;
	o->OnMouseMove(x,y);

	return;
}

void 
EMouse::ReshapeFunction(int w, int h)
{
	ObjectManipulator*o = &objectManipulator;

	if(o==0)
		return;
	o->OnSize(w,h);

	return;
}

void 
EMouse::Enter(void)
{
	EnterCriticalSection(&GlobalCriticalSection);
}

void 
EMouse::Leave(void)
{
	LeaveCriticalSection(&GlobalCriticalSection);
}
