/** @file
 *  @brief This file is implimentaion of class EPhantom.
 */
#include "EPhantom.h"
#include <HD/hd.h>
#include <stdio.h>

static const BReal forceMax=10;
static const BReal frequency=1000;
static const BReal defaultscale=1.0;

static HHD					ghHD = HD_INVALID_HANDLE;
static HDSchedulerHandle	gSchedulerCallback = HD_INVALID_HANDLE;
int DoCalibration(void);

static void 
m16to9(BReal m[9], double t[16])
{
	m[0]=static_cast<BReal>(t[0]);m[3]=static_cast<BReal>(t[1]);m[6]=static_cast<BReal>(t[ 2]);
	m[1]=static_cast<BReal>(t[4]);m[4]=static_cast<BReal>(t[5]);m[7]=static_cast<BReal>(t[ 6]);
	m[2]=static_cast<BReal>(t[8]);m[5]=static_cast<BReal>(t[9]);m[8]=static_cast<BReal>(t[10]);
}

HDCallbackCode HDCALLBACK CallBackFunctionHaptics(void *data)
{
	EPhantom* me = reinterpret_cast<EPhantom*>(data);

	HHD hHD;
	if( hHD= hdGetCurrentDevice() == HD_INVALID_HANDLE )
		printf("HD_INVALID_HANDLE\n");

	//From here(force frame)
	hdBeginFrame(hHD);
	me->CallBackHpFunction();
	hdEndFrame(hHD);
	//end(force frame)

	HDErrorInfo error;
	if (HD_DEVICE_ERROR(error = hdGetError()))
		return HD_CALLBACK_DONE;

	return HD_CALLBACK_CONTINUE;
}


EPhantom::EPhantom()
{
	center_=0;
	origin_=0;
	scale_.x=defaultscale;
	scale_.y=defaultscale;
	scale_.z=defaultscale;

	orientation_.Identity();
	orientation0_.Identity();
	centerOrientation_.Identity();
	centerOrientation0_.Identity();

	position_=0;
	position0_=0;

	last_state_=0;
}

EPhantom::~EPhantom()
{
}

void 
EPhantom::_GetOrientation(BQuaternion&q)
{
	double t[16];
	hdGetDoublev(HD_CURRENT_TRANSFORM,t);
	BReal m[9];
	m16to9(m, t);
	q.SetMatrix0(m);

	return;
}

void 
EPhantom::_GetOrientation(BQuaternion&q,
			   BQuaternion&c0)
{
	double t[16];
	hdGetDoublev(HD_CURRENT_TRANSFORM,t);
	BReal m[9];
	m16to9(m,t);
	q.SetMatrix0(m);
	BReal a[3],b[3];
	q.GetYXZAngle(a);
	c0.GetYXZAngle(b);
	q.SetYXZAngle(a[0]-b[0],
			a[1]-b[1],
			a[2]-b[2]);

	return;
}

void 
EPhantom::SetForce(BVector3&t)
{
	BVector3	force=t;
	BVector3 u;
	u.Set(t.x,t.y,t.z);
	BReal a;a=u.Norm();
	if(a>forceMax)
		u*=forceMax/a;

	HDdouble phantomForce[3];
	HDdouble phantomForce0[3];
	phantomForce[0]=u.x;
	phantomForce[1]=u.y;
	phantomForce[2]=u.z;
	for(;;)
	{
		hdSetDoublev(HD_CURRENT_FORCE, phantomForce);
		hdGetDoublev(HD_CURRENT_FORCE, phantomForce0);
		{
			BVector3 t;
			t.Set(	static_cast<BReal>(phantomForce[0]-phantomForce0[0]),
					static_cast<BReal>(phantomForce[1]-phantomForce0[1]),
					static_cast<BReal>(phantomForce[2]-phantomForce0[2]));
			if(t.Norm()<1e-3)
				break;
		}
	}

	return;
}

void 
EPhantom::SetTorque(BVector3&t)
{
	BVector3 torque=t;
	HDdouble phantomForce[3];
	phantomForce[0]=t.x;
	phantomForce[1]=t.y;
	phantomForce[2]=t.z;
	hdSetDoublev(HD_CURRENT_TORQUE, phantomForce);

	return;
}

void 
EPhantom::SetScale(BReal s)
{
	scale_.x=s;
	scale_.y=s;
	scale_.z=s;

	return;
}

void
EPhantom::SetOrientation(const BQuaternion& q)
{
	BQuaternion t;
	orientation0_=q;
	_GetOrientation(t,centerOrientation_);
	centerOrientation0_=t;
	centerOrientation0_.Normalize();

	return;
}

void 
EPhantom::GetPosition(BVector3& position)
{
	int i=0;
	HDdouble phantomPos[3];
	for(;;)
	{
		hdGetDoublev(HD_CURRENT_POSITION, phantomPos);
		
		position.x=static_cast<BReal>(phantomPos[0])-center_.x;
		position.y=static_cast<BReal>(phantomPos[1])-center_.y;
		position.z=static_cast<BReal>(phantomPos[2])-center_.z;
		Convert(position);

		BVector3 t;
		t=position-position0_;
		if(t.Norm()/scale_.x<10)
			break;
		if(i>10)
			break;
		i++;
	}

	position0_=position_;
	position_=position;

	return;
}

void 
EPhantom::GetOrientation(BQuaternion& q)
{
	BQuaternion t;
	q.Identity();
	_GetOrientation(t,centerOrientation_);
	q=t;
	{
		CoordinateMatrix t;
		BVector3 ex,ey,ez;
		ex.Set(1,0,0);
		ey.Set(0,1,0);
		ez.Set(0,0,1);
		BVector3 fx,fy,fz;
		t.Set(orientation0_);
		t.Convert(fx,ex);
		t.Convert(fy,ey);
		t.Convert(fz,ez);
		t.Set(centerOrientation0_);
		t.InverseRotate(ex,fx);
		t.InverseRotate(ey,fy);
		t.InverseRotate(ez,fz);
		t.Set(q);
		t.Convert(fx,ex);
		t.Convert(fy,ey);
		t.Convert(fz,ez);
		
		ex=fx;ey=fy;ez=fz;
		BReal m[9];
		m[0]=ex.x;m[3]=ex.y;m[6]=ex.z;
		m[1]=ey.x;m[4]=ey.y;m[7]=ey.z;
		m[2]=ez.x;m[5]=ez.y;m[8]=ez.z;
		q.SetMatrix0(m);
	}
	q.Normalize();
	orientation_=q;

	return;
}

int 
EPhantom::GetButton(int n)
{
	int current_state, return_val;
	static int button[4]={	HD_DEVICE_BUTTON_1,HD_DEVICE_BUTTON_2,
							HD_DEVICE_BUTTON_3,HD_DEVICE_BUTTON_4};
	hdGetIntegerv(HD_CURRENT_BUTTONS, &current_state);

	if( !last_state_ && (current_state==button[n]) )
		return_val = EH_BUTTON_DOWN;
	else
		return_val = EH_BUTTON_OFF;

	last_state_ = current_state;

	return return_val;
}

bool 
EPhantom::Initialize(void)
{
	if(DoCalibration()!=0)
		return false;

	HDErrorInfo error;
	ghHD = hdInitDevice(HD_DEFAULT_DEVICE);
	if (HD_DEVICE_ERROR(error = hdGetError()))
	{
		fprintf(stderr, "Failed to initialize haptic device. \n Press any key to quit.\n");
		getchar();
		return false;
	}

	hdEnable(HD_FORCE_OUTPUT);
	hdEnable(HD_MAX_FORCE_CLAMPING);

	//Start Scheduler
	hdStartScheduler();
	if (HD_DEVICE_ERROR(error = hdGetError()))
	{
		fprintf(stderr, "Failed to start scheduler. \n Press any key to quit.\n");
		getchar();
		return false;
	}

	gSchedulerCallback = hdScheduleAsynchronous(
		CallBackFunctionHaptics, this,	HD_DEFAULT_SCHEDULER_PRIORITY);

	return true;
}

bool 
EPhantom::Stop(void)
{
	hdDisable(HD_FORCE_OUTPUT);
	hdDisable(HD_MAX_FORCE_CLAMPING);

	//Stop Scheduler
	hdStopScheduler();
	hdUnschedule(gSchedulerCallback);
	hdDisableDevice(hdGetCurrentDevice());

	return true;
}

void
EPhantom::CallBackHpFunction(void)
{
	if(callBackHpFunction)
		callBackHpFunction();

	return;
}

void
EPhantom::CallBackStFunction(void)
{
	if(callBackStFunction)
		callBackStFunction();

	return;
}

void
EPhantom::CallBackKbFunction(void)
{
	if(callBackKbFunction)
		callBackKbFunction(key_);

	return;
}

void 
EPhantom::SetCallBackHpFunction(void(*hp)(void))
{
	callBackHpFunction=hp;

	return;
}

HDCallbackCode HDCALLBACK StateCallback(void *pUserData)
{
	EPhantom* me = reinterpret_cast<EPhantom*>(pUserData);
	me->CallBackStFunction();

    /* execute this only once. */
    return HD_CALLBACK_DONE;
}

void 
EPhantom::SetCallBackStFunction(void(*st)(void))
{
	callBackStFunction=st;
	hdScheduleSynchronous(StateCallback,
							this, HD_MIN_SCHEDULER_PRIORITY);
	return;
}

HDCallbackCode HDCALLBACK KeyCallback(void *pUserData)
{
	EPhantom* me = reinterpret_cast<EPhantom*>(pUserData);
	me->CallBackKbFunction();

	/* execute this only once. */
	return HD_CALLBACK_DONE;
}

void 
EPhantom::SetCallBackKbFunction(void(*kb)(unsigned char), unsigned char key)
{
	callBackKbFunction=kb;
	key_=key;
	hdScheduleSynchronous(KeyCallback,
							this, HD_MIN_SCHEDULER_PRIORITY);
	return;
}

void 
EPhantom::WaitForUserAdmit()
{
	printf("Hold the haptic device handle in the neutral position.\n");
	printf("Press enter to continue\n");
	char s[100];
	fgets(s,100,stdin);

	return;
}
