/** @file
 *  @brief This file is implimentaion of class EasyDevice.
 */
#include "EasyDevice.h"

#ifdef USE_PHANTOM
#include "EPhantom.h"
#else
#include "EMouse.h"
#endif

/************************************* class EasyDevice::Impl *******************************/

/** @brief Default Constructor
 */
EasyDevice::Impl::Impl()
{
}

/** @brief Destructor
 */
EasyDevice::Impl::~Impl()
{
}

void
EasyDevice::Impl::SetForce(BVector3&t)
{
	return;
}

void
EasyDevice::Impl::SetTorque(BVector3&t)
{
	return;
}

int
EasyDevice::Impl::GetButton(int n)
{
	return 0;
}

void
EasyDevice::Impl::WaitForUserAdmit()
{
	return;
}

void 
EasyDevice::Impl::MouseFunction(int button,int state, int x, int y)
{
	return;
}

void 
EasyDevice::Impl::MotionFunction(int x, int y)
{
	return;
}

void 
EasyDevice::Impl::ReshapeFunction(int w, int h)
{
	return;
}



/************************************* class EasyDevice *************************************/
EasyDevice::EasyDevice(void)
{
#ifdef USE_PHANTOM
	impl_ = new EPhantom();
#else
	impl_ = new EMouse();
#endif
}

EasyDevice::~EasyDevice(void)
{
    delete impl_;
}


bool
EasyDevice::Start(void)
{
    return impl_->Initialize();
}

bool
EasyDevice::Stop(void)
{
    return impl_->Stop();
}


void
EasyDevice::SetForce(const float force[3])
{
    BVector3 t;t=force;
    impl_->SetForce(t);
	return;
}

void
EasyDevice::SetForce(const double force[3])
{
    BVector3 t;t=force;
    impl_->SetForce(t);
	return;
}

void
EasyDevice::SetTorque(const float torque[3])
{
    BVector3 t;t=torque;
    impl_->SetTorque(t);
	return;
}

void
EasyDevice::SetTorque(const double torque[3])
{
    BVector3 t;t=torque;
    impl_->SetTorque(t);
	return;
}

void
EasyDevice::SetScale(float scale)
{
    impl_->SetScale(scale);
	return;
}

void
EasyDevice::SetScale(double scale)
{
    impl_->SetScale(static_cast<BReal>(scale));
	return;
}

void
EasyDevice::SetOrientation(const float orientation[4])
{
    BQuaternion t;t=orientation;
    impl_->SetOrientation(t);
	return;
}

void
EasyDevice::SetOrientation(const double orientation[4])
{
    BQuaternion t;t=orientation;
    impl_->SetOrientation(t);
	return;
}


void
EasyDevice::GetPosition(float position[3])
{
    BVector3 t;
    impl_->GetPosition(t);
    t.GetValue(position);
	return;
}

void
EasyDevice::GetPosition(double position[3])
{
    BVector3 t;
    impl_->GetPosition(t);
    t.GetValue(position);
	return;
}

void
EasyDevice::GetOrientation(float orientation[4])
{
    BQuaternion t;
    impl_->GetOrientation(t);
    t.GetValue(orientation);
	return;
}

void
EasyDevice::GetOrientation(double orientation[4])
{
    BQuaternion t;
    impl_->GetOrientation(t);
    t.GetValue(orientation);
	return;
}

void
EasyDevice::GetLastMatrix(float matrix[16])
{
    BVector3 p;
    BQuaternion q;
    impl_->GetLastPosition(p);
    impl_->GetLastOrientation(q);
    CoordinateMatrix c;c.Set(p,q);
    c.GetGLMatrix(matrix);
	return;
}

void
EasyDevice::GetLastMatrix(double matrix[16])
{
    BVector3 p;
    BQuaternion q;
    impl_->GetLastPosition(p);
    impl_->GetLastOrientation(q);
    CoordinateMatrix c;c.Set(p,q);
    c.GetGLMatrix(matrix);
	return;
}

void
EasyDevice::SetHapticFunction(void(*hpfunction)(void))
{
    impl_->SetCallBackHpFunction(hpfunction);
	return;
}

void
EasyDevice::SetStatusFunction(void(*stfunction)(void))
{
	impl_->SetCallBackStFunction(stfunction);
	return;
}

void
EasyDevice::SetKeyboardFunction(void(*kbfunction)(unsigned char), unsigned char key)
{
	impl_->SetCallBackKbFunction(kbfunction, key);
	return;
}


void
EasyDevice::MouseFunction(int button,int state, int x, int y)
{
    impl_->MouseFunction(button,state,x,y);
	return;
}

void 
EasyDevice::MotionFunction(int x, int y)
{
    impl_->MotionFunction(x,y);
	return;
}

void
EasyDevice::ReshapeFunction(int w, int h)
{
    impl_->ReshapeFunction(w,h);
	return;
}

int
EasyDevice::GetButton(int n)
{
    return impl_->GetButton(n);
}

void
EasyDevice::WaitForUserAdmit()
{
    impl_->WaitForUserAdmit();
	return;
}
